@@ -169,7 +169,7 @@ setMethod("export", c("GenomicRanges", "BEDFile"),
             on.exit(options(scipen = scipen))
             file <- con
             con <- connection(con, if (append) "a" else "w")
-            on.exit(release(con))
+            on.exit(release(con), add=TRUE)
             write.table(df, con, sep = "\t", col.names = FALSE,
                         row.names = FALSE, quote = FALSE, na = ".")
             release(con)
@@ -324,8 +324,9 @@ setMethod("import", "BEDFile",
                              use.names=FALSE)
               cols <- matrix(as.integer(cols), 3)
               color <- rep(NA, nrow(bed))
-              color[spec] <- rgb(cols[1,], cols[2,], cols[3,], max = 255)
-              bed$itemRgb <- color              
+              color[spec] <- rgb(cols[1,], cols[2,], cols[3,],
+                                 maxColorValue = 255L)
+              bed$itemRgb <- color
             }
             fromCSV <- function(b) {
               as.integer(unlist(strsplit(b, ",", fixed = TRUE)))
@@ -366,7 +367,7 @@ setMethod("import", "BED15File",
             if (!missing(format))
               checkArgFormat(con, format)
             if (is.null(trackLine))
-              return(import.ucsc(con, TRUE, genome = genome,
+              return(import.ucsc(con, drop = TRUE, genome = genome,
                                  which = which))
             bed <- callNextMethod()
             if (!length(bed))
@@ -407,7 +408,9 @@ setMethod("export.bed15", "ANY",
 
 ### FIXME: dispatch will break when 'object' is a UCSCData
 ### Possible solution: just merge this code with the main BEDFile method?
-setMethod("export", c("GenomicRanges", "BED15File"),
+setMethods("export",
+           list(c("GenomicRanges", "BED15File"),
+                c("UCSCData", "BED15File")),
           function(object, con, format, expNames = NULL, trackLine = NULL, ...)
           {
             if (!missing(format))
@@ -423,16 +426,19 @@ setMethod("export", c("GenomicRanges", "BED15File"),
             object$expCount <- rep(length(expNames), length(object))
             object$expIds <- rep(paste(seq_along(expNames)-1, collapse=","),
                                  length(object))
-            scores <- as.list(unlist(values(object[,expNames])))
+            scores <- as.list(mcols(object)[expNames])
             scores <- do.call(paste, c(scores, sep = ","))
             scores <- gsub("NA", "-10000", scores, fixed=TRUE)
+            if (length(scores) == 0L) {
+                scores <- rep("", length(object))
+            }
             object$expScores <- scores
             callNextMethod(object, con, ...)
           })
 
 setClass("Bed15TrackLine",
          representation(expStep = "numeric", expScale = "numeric",
-                        expNames = "characterORNULL"),
+                        expNames = "character_OR_NULL"),
          prototype(expStep = 0.5, expScale = 3.0), 
          contains = "BasicTrackLine") # not sure which fields work
 
@@ -577,6 +583,7 @@ setMethod("export", c("Pairs", "BEDPEFile"),
             on.exit(options(scipen = scipen))
             file <- con
             con <- connection(con, if (append) "a" else "w")
+            on.exit(release(con), add=TRUE)
             write.table(df, con, sep = "\t", col.names = FALSE,
                         row.names = FALSE, quote = FALSE, na = ".")
             release(con)
