@@ -37,7 +37,8 @@ setMethod("initialize", "UCSCSession",
 setMethod("seqlengths", "UCSCSession", function(x) {
   query <- ucscTableQuery(x, range = GRanges(), table = "chromInfo")
   chromInfo <- getTable(query)
-  setNames(chromInfo$size, chromInfo$chrom)
+  ans <- setNames(chromInfo$size, chromInfo$chrom)
+  ans[sortSeqlevels(names(ans))]
 })
 
 setMethod("seqnames", "UCSCSession", function(x) names(seqlengths(x)))
@@ -145,12 +146,12 @@ GRangesForUCSCGenome <- function(genome, chrom = NULL, ranges = NULL, ...)
 ## context for querying UCSC tables
 setClass("UCSCTableQuery",
          representation(session = "UCSCSession",
-                        track = "characterORNULL",
-                        table = "characterORNULL",
+                        track = "character_OR_NULL",
+                        table = "character_OR_NULL",
                         range = "GRanges",
-                        outputType = "characterORNULL",
-                        NAMES = "characterORNULL",
-                        intersectTrack = "characterORNULL"))
+                        outputType = "character_OR_NULL",
+                        NAMES = "character_OR_NULL",
+                        intersectTrack = "character_OR_NULL"))
 
 setMethod("show", "UCSCTableQuery",
           function(object) {
@@ -235,7 +236,7 @@ setMethod("intersectTrack", "UCSCTableQuery", function(x) x@intersectTrack)
 setGeneric("intersectTrack<-", function(x, ..., value)
            standardGeneric("intersectTrack<-"))
 setReplaceMethod("intersectTrack", "UCSCTableQuery", function(x, value) {
-  x@intersectTrack <- normArgTrack(value, x, TRUE)
+  x@intersectTrack <- normArgTrack(value, x)
   x
 })
 
@@ -275,7 +276,7 @@ setMethod("ucscTableQuery", "UCSCSession",
           function(x, track = NULL, range = genome(x), table = NULL,
                    names = NULL, intersectTrack = NULL)
           {
-            if (!is(names, "characterORNULL"))
+            if (!is(names, "character_OR_NULL"))
               stop("'names' must be 'NULL' or a character vector")
             ## only inherit the genome from the session
             range <- normTableQueryRange(range, x)
@@ -1518,13 +1519,7 @@ setMethod("ucscTrackModes", "ucscTracks",
 
 ucscGenomes <- function(organism=FALSE) {
   url <- "http://genome.ucsc.edu/FAQ/FAQreleases"
-  doc <- httpGet(url)
-  table <- getNodeSet(doc, "//table[@class='descTbl']")[[1L]]
-  species <- sapply(getNodeSet(table, "tr/td[1]"), xmlValue)
-  dbs <- sapply(getNodeSet(table, "tr/td[2]"), xmlValue)
-  dates <- sapply(getNodeSet(table, "tr/td[3]"), xmlValue)
-  nms <- sapply(getNodeSet(table, "tr/td[4]"), xmlValue)
-  status <- sapply(getNodeSet(table, "tr/td[5]"), xmlValue)
+  df <- readHTMLTable(url)[[1L]][c(2,1,3,4,5)]
   .cleanTableCells <- function(x)
   {
     x <- sub("^ *", "", x)
@@ -1537,23 +1532,23 @@ ucscGenomes <- function(organism=FALSE) {
     x[is_empty_cell] <- ""
     x
   }
-  df <- data.frame(db=.cleanTableCells(dbs),
-                   species=.cleanTableCells(species),
-                   date=.cleanTableCells(dates),
-                   name=.cleanTableCells(nms),
-                   status=.cleanTableCells(status),
-                   stringsAsFactors=FALSE)
   COLS <- c("UCSC VERSION", "SPECIES", "RELEASE DATE", "RELEASE NAME", "STATUS")
-  if (!identical(as.character(df[1L, ]), COLS))
+  if (!identical(names(df), COLS))
     stop("table of UCSC genome releases (found at ", url, "#release1) ",
          "doesn't have expected columns ", paste(COLS, collapse=", ")) 
-  df <- df[-1L, ]
+  ## readHTMLTable is returning one NA row for some reason
+  names(df) <- c("db","species","date","name","status")
+  df <- df[!apply(df, 1, function(x) all(is.na(x))),]
   df <- df[df$db != "" , ]
   not_empty <- df$species != ""
   df$species <- rep.int(df$species[not_empty], diff(which(c(not_empty, TRUE))))
   df <- df[df$status == "Available", -5L]
-  if (organism)
-    df$organism <- .getOrganism(df$db)
+  if (organism) {
+    df$organism <- NA_character_
+    org <- lapply(as.character(df$db), function(xx) 
+                  unique(suppressWarnings(mapGenomeBuilds(xx)$organism)))
+    df$organism[!sapply(org, is.null)] <- unlist(org) 
+  }
   rownames(df) <- NULL
   df
 }
